with System; use System;
with Ada.Unchecked_Conversion;
with Grt.Types; use Grt.Types;
with Grt.Rtis; use Grt.Rtis;
with Grt.Rtis_Addr; use Grt.Rtis_Addr;
with Grt.Rtis_Utils; use Grt.Rtis_Utils;
with Grt.Rtis; use Grt.Rtis;
with Grt.Astdio; use Grt.Astdio;
with Grt.Errors; use Grt.Errors;
pragma Elaborate_All (Grt.Rtis_Utils);
with Grt.Vstrings; use Grt.Vstrings;
with Grt.Stdio; use Grt.Stdio;
with Grt.Signals; use Grt.Signals;
with Grt.Options;

package body Grt.Disp_Signals is
   procedure Disp_Context (Ctxt : Rti_Context)
   is
      Blk : Ghdl_Rtin_Block_Acc;
      Nctxt : Rti_Context;
   begin
      Blk := To_Ghdl_Rtin_Block_Acc (Ctxt.Block);
      case Blk.Common.Kind is
         when Ghdl_Rtik_Block
           | Ghdl_Rtik_Process =>
            Nctxt := Get_Parent_Context (Ctxt);
            Disp_Context (Nctxt);
            Put ('.');
            Put (Blk.Name);
         when Ghdl_Rtik_Entity =>
            Put (Blk.Name);
         when Ghdl_Rtik_Architecture =>
            Nctxt := Get_Parent_Context (Ctxt);
            Disp_Context (Nctxt);
            Put ('(');
            Put (Blk.Name);
            Put (')');
         when others =>
            Internal_Error ("disp_context");
      end case;
   end Disp_Context;

   --  Disp transaction TRANS from signal SIG.
   procedure Disp_Transaction (Trans : Transaction_Acc;
                               Sig_Type : Ghdl_Rti_Access)
   is
      T : Transaction_Acc;
   begin
      T := Trans;
      loop
         case T.Kind is
            when Trans_Value =>
               Disp_Value (stdout, T.Val, Sig_Type);
            when Trans_Null =>
               Put ("NULL");
            when Trans_Error =>
               Put ("ERROR");
         end case;
         Put ("@");
         Disp_Time (T.Time);
         T := T.Next;
         exit when T = null;
         Put (", ");
      end loop;
   end Disp_Transaction;

   procedure Disp_Simple_Signal
     (Sig : Ghdl_Signal_Ptr; Sig_Type : Ghdl_Rti_Access; Sources : Boolean)
   is
   begin
      Put (' ');
      Put (stdout, Sig.all'Address);
      Put (' ');
      case Sig.Mode is
         when Mode_B2 =>
            Put (" b2");
         when Mode_E8 =>
            Put (" e8");
         when Mode_E32 =>
            Put ("e32");
         when Mode_I32 =>
            Put ("i32");
         when Mode_P64 =>
            Put ("p64");
         when Mode_F64 =>
            Put ("f64");
      end case;
      Put (' ');
      if Sig.Active then
         Put ('A');
      else
         Put ('-');
      end if;
      if Sig.Event then
         Put ('E');
      else
         Put ('-');
      end if;
      if Sig.Effective /= null then
         Put ('e');
      else
         Put ('-');
      end if;
      if Boolean'(True) then
         Put (" last_event=");
         Disp_Time (Sig.Last_Event);
         Put (" last_active=");
         Disp_Time (Sig.Last_Active);
      end if;
      Put (" val=");
      Disp_Value (stdout, Sig.Value, Sig_Type);
      Put ("; drv=");
      Disp_Value (stdout, Sig.Driving_Value, Sig_Type);
      if Sources then
         if Sig.Nbr_Ports > 0 then
            Put (';');
            Put_I32 (stdout, Ghdl_I32 (Sig.Nbr_Ports));
            Put (" ports");
         end if;
         if Sig.Nbr_Drivers = 0 then
            Put ("; no driver");
         elsif Sig.Nbr_Drivers = 1 then
            Put ("; trans=");
            Disp_Transaction (Sig.Drivers (0).First_Trans, Sig_Type);
         else
            for I in 0 .. Sig.Nbr_Drivers - 1 loop
               New_Line;
               Put ("   ");
               Disp_Transaction (Sig.Drivers (I).First_Trans, Sig_Type);
            end loop;
         end if;
      end if;
      New_Line;
   end Disp_Simple_Signal;

   procedure Disp_Scalar_Signal (Val_Addr : Address;
                                 Val_Name : Vstring;
                                 Val_Type : Ghdl_Rti_Access)
   is
      function To_Ghdl_Signal_Ptr is new Ada.Unchecked_Conversion
        (Source => Address, Target => Ghdl_Signal_Ptr);
   begin
      Put (stdout, Val_Name);
      Disp_Simple_Signal (To_Ghdl_Signal_Ptr (To_Addr_Acc (Val_Addr).all),
                          Val_Type, Options.Disp_Sources);
   end Disp_Scalar_Signal;

   procedure Foreach_Scalar_Signal is new
     Foreach_Scalar (Process => Disp_Scalar_Signal);

   function Disp_Signal (Ctxt : Rti_Context;
                         Obj : Ghdl_Rti_Access)
                        return Traverse_Result
   is
   begin
      case Obj.Kind is
         when Ghdl_Rtik_Signal
           | Ghdl_Rtik_Port
           | Ghdl_Rtik_Guard =>
            declare
               Sig : Ghdl_Rtin_Object_Acc;
            begin
               Sig := To_Ghdl_Rtin_Object_Acc (Obj);
               Put (stdout, Ctxt);
               Put (".");
               Put (Sig.Name);
               New_Line;
               Foreach_Scalar_Signal
                 (Ctxt, Sig.Obj_Type,
                  Loc_To_Addr (Sig.Common.Depth, Sig.Loc, Ctxt), True);
            end;
         when others =>
            null;
      end case;
      return Traverse_Ok;
   end Disp_Signal;

   function Disp_All_Signals is new Traverse_Blocks (Process => Disp_Signal);

   procedure Disp_All_Signals
   is
      Res : Traverse_Result;
   begin
      Res := Disp_All_Signals (Get_Top_Context);
   end Disp_All_Signals;

end Grt.Disp_Signals;
